/*
 * Copyright (C) 2005-2006 Kouji Sugisawa. All rights reserved.
 */

package jp.sourceforge.livez.dao.hibernate;

import java.util.List;

import org.hibernate.LockMode;
import org.hibernate.Query;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.ObjectRetrievalFailureException;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

import jp.sourceforge.livez.dao.EZweb;
import jp.sourceforge.livez.dao.EZwebDAO;

/**
 * EZwebDAO  Hibernate 3 ŎNXłB
 * 
 * @author V _
 */
public final class EZwebDaoImpl extends HibernateDaoSupport implements EZwebDAO {

	/**
	 * ftHgRXgN^łB
	 */
	public EZwebDaoImpl() {}

	/*
	 * @see jp.sourceforge.livez.dao.EZwebDAO#loadAll()
	 */
	@SuppressWarnings("unchecked")
	public List<EZweb> loadAll() throws DataAccessException {
		HibernateTemplate template = getHibernateTemplate();
		return template.loadAll(EZweb.class);
	}

	/*
	 * @see jp.sourceforge.livez.dao.EZwebDAO#load(java.lang.String)
	 */
	@SuppressWarnings("unchecked")
	public EZweb load(final String deviceType) throws DataAccessException {
		HibernateTemplate template = getHibernateTemplate();
		try {
			return (EZweb) template.load(EZweb.class, deviceType);
		} catch (ObjectRetrievalFailureException e) {
			return null;
		}
	}

	/*
	 * @see jp.sourceforge.livez.dao.EZwebDAO#save(jp.sourceforge.livez.dao.EZweb)
	 */
	public void save(final EZweb ezweb) throws DataAccessException {
		HibernateTemplate template = getHibernateTemplate();
		template.save(ezweb);
	}

	/*
	 * @see jp.sourceforge.livez.dao.EZwebDAO#update(jp.sourceforge.livez.dao.EZweb, java.lang.String)
	 */
	public void update(final EZweb ezweb, final String deviceType) throws DataAccessException {
		if (!deviceType.equals(ezweb.getDeviceType())) {
			Query query = getSession().createQuery("UPDATE EZweb SET deviceType = ? WHERE deviceType = ?");
			query.setString(0, ezweb.getDeviceType());
			query.setString(1, deviceType);
			query.executeUpdate();
		}

		HibernateTemplate template = getHibernateTemplate();
		template.update(ezweb, LockMode.UPGRADE_NOWAIT);
	}

	/*
	 * @see jp.sourceforge.livez.dao.EZwebDAO#delete(java.lang.String)
	 */
	public void delete(final String deviceType) throws DataAccessException {
		HibernateTemplate template = getHibernateTemplate();
		EZweb ezweb = (EZweb) template.load(EZweb.class, deviceType);
		template.delete(ezweb, LockMode.UPGRADE_NOWAIT);
	}

}
