/*
 * Copyright (C) 2006 Kouji Sugisawa. All rights reserved.
 */

package jp.sourceforge.livez.pages;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import jp.sourceforge.livez.pages.AbstractPageHandler;

/**
 * @author V _
 */
final class FramePageHandler extends AbstractPageHandler {

	@Override
	protected void doStart(
		final String uri,
		final String localName,
		final String qName,
		final Attributes attributes)
		throws SAXException {

		if (qName.equalsIgnoreCase("body")) {
			page.append(renderStyle(attributes));
			page.append("<div");
			page.append(renderDivStyle(attributes));
			page.append('>');
		} else
			super.doStart(uri, localName, qName, attributes);
	}

	private static String renderStyle(final Attributes attributes) {
		StringBuilder sb = new StringBuilder();
		String s;

		s = attributes.getValue("link");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("a:link{color:");
			sb.append(s);
			sb.append('}');
		}

		s = attributes.getValue("alink");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("a:focus{color:");
			sb.append(s);
			sb.append('}');
		}

		s = attributes.getValue("vlink");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("a:visited{color:");
			sb.append(s);
			sb.append('}');
		}

		if (sb.length() > 0) {
			sb.insert(0, "<style type=\"text/css\">");
			sb.append("</style>");
		}

		return sb.toString();
	}

	private static String renderDivStyle(final Attributes attributes) {
		StringBuilder sb = new StringBuilder();
		String s;

		s = attributes.getValue("text");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("color:");
			sb.append(s);
		}

		s = attributes.getValue("bgcolor");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("background-color:");
			sb.append(s);
		}

		s = attributes.getValue("background");
		if (s != null) {
			if (sb.length() > 0)
				sb.append(';');
			sb.append("background-image:url(");
			sb.append(s);
			sb.append(')');
		}

		if (sb.length() > 0) {
			sb.insert(0, " style=\"");
			sb.append('"');
		}

		return sb.toString();
	}

	@Override
	protected void doEnd(
		final String uri,
		final String localName,
		final String qName)
		throws SAXException {

		if (qName.equalsIgnoreCase("body"))
			page.append("</div>");
		else
			super.doEnd(uri, localName, qName);
	}

}
